package com.bsn.mes.pro.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.annotation.Log;
import com.bsn.common.core.controller.BaseController;
import com.bsn.common.core.domain.AjaxResult;
import com.bsn.common.core.page.TableDataInfo;
import com.bsn.common.enums.BusinessType;
import com.bsn.common.utils.data.ExceptionUtil;
import com.bsn.common.utils.data.StringUtils;
import com.bsn.common.utils.poi.ExcelUtil;
import com.bsn.mes.constant.TaskWorkunitStatusEnum;
import com.bsn.mes.pro.domain.ProFeedback;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderSoSizeItem;
import com.bsn.mes.pro.domain.vo.ProFeedbackVo;
import com.bsn.mes.pro.domain.vo.proWorkOrder.WorkOrderSoSizeItemStandard;
import com.bsn.mes.pro.mapper.ProFeedbackMapper;
import com.bsn.mes.pro.service.IProFeedbackService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderSoSizeItemService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 生产报工记录Controller
 *
 * @date 2022-07-10
 */
@RestController
@RequestMapping("/mes/pro/feedback")
public class ProFeedbackController extends BaseController {
    @Autowired
    private IProFeedbackService proFeedbackService;

    @Resource
    private ProFeedbackMapper proFeedbackMapper;

    @Autowired
    private IProWorkOrderSoSizeItemService proWorkOrderSoSizeItemService;


    /**
     * 查询生产报工记录列表
     */
    @GetMapping("/list")
    public TableDataInfo<ProFeedbackVo> list(ProFeedback proFeedback) {
        startPage();
        QueryWrapper<ProFeedback> query = new QueryWrapper<>();
        query.eq(StringUtils.isNotEmpty(proFeedback.getFeedbackType()), "f.feedback_type", proFeedback.getFeedbackType());
        query.eq(StringUtils.isNotEmpty(proFeedback.getStatus()), "f.status", proFeedback.getStatus());
        query.eq(StringUtils.isNotEmpty(proFeedback.getAssistProcessCode()), "ap.assist_process_code", proFeedback.getAssistProcessCode());
        query.eq(StringUtils.isNotEmpty(proFeedback.getWorkorderCode()), "f.workorder_code", proFeedback.getWorkorderCode());
        query.eq(StringUtils.isNotEmpty(proFeedback.getWorkstationName()), "f.workstation_name", proFeedback.getWorkstationName());
        query.eq(StringUtils.isNotEmpty(proFeedback.getItemName()), "it.item_name", proFeedback.getItemName());
        query.eq(StringUtils.isNotEmpty(proFeedback.getWorkorderCode()), "f.workorder_code", proFeedback.getWorkorderCode());
        query.eq(StringUtils.isNotEmpty(proFeedback.getSapItemCode()), "it.sap_item_code", proFeedback.getSapItemCode());
        query.eq(StringUtils.isNotEmpty(proFeedback.getUserId()), "pww.user_id", proFeedback.getUserId());
        query.eq(StringUtils.isNotEmpty(proFeedback.getWorkunitId()), "tw.workunit_id", proFeedback.getWorkunitId());
        query.eq(StringUtils.isNotEmpty(proFeedback.getTaskWorkunitId()), "tw.task_workunit_id", proFeedback.getTaskWorkunitId());
        query.between(StringUtils.isNotEmpty(proFeedback.getCreateStartTime()) && StringUtils.isNotEmpty(proFeedback.getCreateEndTime()), "it.sap_item_code", proFeedback.getCreateStartTime(), proFeedback.getCreateEndTime());
        query.like(StringUtils.isNotEmpty(proFeedback.getNickName()), "f.nick_name", proFeedback.getNickName());
        query.notIn("tw.status", TaskWorkunitStatusEnum.CLOSE.getStatus());
        List<ProFeedbackVo> list = proFeedbackService.queryProFeedbackListJoinTaskWorkUnit(query);
        List<Long> sizeList = list.stream().map(ProFeedbackVo::getWorkorderSoSizeItemId).filter(Objects::nonNull).collect(Collectors.toList());
        //查询关联的标准SIZE
        if (CollectionUtil.isNotEmpty(sizeList)) {
            QueryWrapper<ProWorkOrderSoSizeItem> querySize = new QueryWrapper<>();
            querySize.in("workorder_so_size_item_id", sizeList);
            List<WorkOrderSoSizeItemStandard> sizeStandardList = proWorkOrderSoSizeItemService.selectProWorkOrderSoSizeItemStandardList(querySize);
            list.forEach(s -> sizeStandardList.stream().filter(temp -> temp.getWorkorderSoSizeItemId().equals(s.getWorkorderSoSizeItemId())).forEach(temp -> s.setStandardSize(temp.getStandardSize())));
        }


        return getDataTable(list);
    }

    /**
     * 导出生产报工记录列表
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:export')")
    @Log(title = "生产报工记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, ProFeedback proFeedback) {
        List<ProFeedback> list = proFeedbackService.selectProFeedbackList(proFeedback);
        ExcelUtil<ProFeedback> util = new ExcelUtil<ProFeedback>(ProFeedback.class);
        util.exportExcel(response, list, "生产报工记录数据");
    }

    /**
     * 获取生产报工记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:query')")
    @GetMapping(value = "/{recordId}")
    public AjaxResult getInfo(@PathVariable("recordId") Long recordId) {
        return AjaxResult.success(proFeedbackService.selectProFeedbackByRecordId(recordId));
    }

    /**
     * 新增生产报工记录
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:add')")
    @Log(title = "生产报工记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody ProFeedback proFeedback) {
        return toAjax(proFeedbackService.addObj(proFeedback));
    }

    /**
     * 修改生产报工记录
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:edit')")
    @Log(title = "生产报工记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody ProFeedback proFeedback) {
        return toAjax(proFeedbackService.updateObj(proFeedback));
    }

    /**
     * 修改生产报工记录
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:edit')")
    @Log(title = "生产报工记录", businessType = BusinessType.UPDATE)
    @PutMapping("/editOut5Min")
    public AjaxResult editOut5Min(@RequestBody ProFeedback proFeedback) {
        ExceptionUtil.checkTruethrowException(((new Date().getTime() - proFeedback.getCreateTime().getTime()) / 1000 / 60) >= 5, "超过5天不能更新");
        return toAjax(proFeedbackService.updateObj(proFeedback));
    }

    /**
     * 删除生产报工记录
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:remove')")
    @Log(title = "生产报工记录", businessType = BusinessType.DELETE)
    @DeleteMapping("/deleteOut5Min/{recordIds}")
    public AjaxResult deleteOut5Min(@PathVariable Long[] recordIds) {
        List<ProFeedback> proFeedbacks = proFeedbackMapper.selectListByQw(new QueryWrapper<ProFeedback>().in("record_id", Arrays.asList(recordIds)));
        for (ProFeedback proFeedback : proFeedbacks) {
            ExceptionUtil.checkTruethrowException(((new Date().getTime() - proFeedback.getCreateTime().getTime()) / 1000 / 60) >= 5, "超过5天不能更新");
        }
        return toAjax(proFeedbackService.deleteProFeedbackByRecordIds(recordIds));
    }

    /**
     * 删除生产报工记录
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:remove')")
    @Log(title = "生产报工记录", businessType = BusinessType.DELETE)
    @DeleteMapping("/{recordIds}")
    public AjaxResult remove(@PathVariable Long[] recordIds) {
        return toAjax(proFeedbackService.deleteProFeedbackByRecordIds(recordIds));
    }


    /**
     * 执行报工
     * 1.更新生产任务和生产工单的进度
     * 2.物料消耗
     * 3.产品产出
     *
     * @param recordId
     * @return
     */
    @PreAuthorize("@ss.hasPermi('mes:pro:feedback:edit')")
    @Log(title = "生产报工执行", businessType = BusinessType.UPDATE)
    @Transactional
    @PutMapping("/{recordId}")
    public AjaxResult execute(@PathVariable("recordId") Long recordId) {
        return toAjax(proFeedbackService.syncSap(recordId));
    }


}
