package com.bsn.mes.pro.controller.mobile;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.annotation.Log;
import com.bsn.common.core.controller.BaseController;
import com.bsn.common.core.domain.AjaxResult;
import com.bsn.common.core.page.TableDataInfo;
import com.bsn.common.enums.BusinessType;
import com.bsn.common.utils.data.StringUtils;
import com.bsn.mes.constant.TaskStatusEnum;
import com.bsn.mes.pro.domain.ProFeedback;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.domain.task.ProTask;
import com.bsn.mes.pro.domain.task.ProTaskIssue;
import com.bsn.mes.pro.domain.task.ProTaskWorkorder;
import com.bsn.mes.pro.domain.vo.ProPackagePrintResult;
import com.bsn.mes.pro.domain.vo.ProTaskQuery;
import com.bsn.mes.pro.domain.vo.ProTaskVo;
import com.bsn.mes.pro.domain.vo.ProWorkorderQuery;
import com.bsn.mes.pro.domain.vo.proWorkOrder.WorkorderPackagePrint;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageAddParams;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageInfo;
import com.bsn.mes.pro.domain.vo.task.FeedbackPackageQueryParams;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkorderMapper;
import com.bsn.mes.pro.mapper.task.ProTaskMapper;
import com.bsn.mes.pro.mapper.task.ProTaskWorkorderMapper;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkorderService;
import com.bsn.mes.pro.service.task.IProTaskIssueService;
import com.bsn.mes.pro.service.task.IProTaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.collectingAndThen;
import static java.util.stream.Collectors.toCollection;

@Api
@RestController
@RequestMapping("/mobile/pro/protask")
public class ProTaskMobController extends BaseController {

    @Autowired
    private IProTaskService proTaskService;

    @Autowired
    private IProTaskIssueService proTaskIssueService;


    @Autowired
    private IProWorkorderService proWorkorderService;


    @Autowired
    private ProWorkorderMapper proWorkorderMapper;

    @Autowired
    private ProTaskWorkorderMapper proTaskWorkorderMapper;


    @Autowired
    private ProTaskMapper proTaskMapper;

    @GetMapping("/getlist")
    public AjaxResult getIssueList(ProTaskIssue proTaskIssue) {
        List<ProTaskIssue> list = proTaskIssueService.selectProTaskIssueList(proTaskIssue);
        return AjaxResult.success(list);
    }

    /**
     * 查询工作站的生产任务
     */
    @ApiOperation("查询状态未完成的生产任务接口")
    @GetMapping("/getTaskList")
    public TableDataInfo<ProTask> list(ProTaskQuery proTask) {
        ProTaskQuery proTaskQuery = new ProTaskQuery();
        BeanUtils.copyProperties(proTask, proTaskQuery);
        //判断前端传来是编号还是工单
        if (StringUtils.isNotEmpty(proTask.getArrangeCode())) {
            if (proTask.getArrangeCode().length() == 12) {
                proTask.setWorkorderCode(proTask.getArrangeCode());
                proTask.setArrangeCode(null);
            }
        }
        List<ProTask> rst = new ArrayList<>();
        List<ProTask> list = new ArrayList<>();
        QueryWrapper<ProTask> query = new QueryWrapper<>();
        List<String> statusList = Arrays.asList(TaskStatusEnum.BEGINNING.getStatus(), TaskStatusEnum.PREPARE.getStatus(),
                TaskStatusEnum.PAUSE.getStatus(), TaskStatusEnum.ERROR_STOP.getStatus());
        query.in("ptw.status", statusList);
        query.eq(proTask.getWorkunitId() != null, "mw.workunit_id", proTask.getWorkunitId());
        query.eq(StringUtils.isNotEmpty(proTask.getArrangeCode()), "t.arrange_code", proTask.getArrangeCode());
        query.gt(proTask.getAllowProduce() != null && proTask.getAllowProduce() == 1, "t.quantity_wait", 0);
        query.orderByAsc("statusIndex").orderByAsc("ptw.schedule_start_date");
        //搜索条件为工单号查询相应任务ID作为查询条件
        if (StringUtils.isNotEmpty(proTask.getWorkorderCode())) {
            QueryWrapper<ProTask> taskQuery = new QueryWrapper<>();
            taskQuery.eq("t3.workorder_code", proTask.getWorkorderCode());
            List<ProTaskVo> tempTaskList = proTaskService.selectProTaskJoinWorkorder(taskQuery);
            if (CollectionUtil.isEmpty(tempTaskList)) {
                return getDataTable(rst);
            } else {
                List<Long> taskIds = tempTaskList.stream().map(ProTaskVo::getTaskId).collect(Collectors.toList());
                query.in("t.task_id", taskIds);
            }
        }
        //判断是否前端有没传来工单或者是编码
        if (StringUtils.isNotEmpty(proTaskQuery.getArrangeCode())) {
            QueryWrapper<ProTask> query2 = new QueryWrapper<>();
            query2.in("ptw.status", statusList);
            query2.eq(proTask.getWorkunitId() != null, "mw.workunit_id", proTask.getWorkunitId());
            query2.gt(proTask.getAllowProduce() != null && proTask.getAllowProduce() == 1, "t.quantity_wait", 0);
            query2.orderByAsc("statusIndex").orderByAsc("ptw.schedule_start_date");

            List<ProTask> proTasks = proTaskService.selectProTaskJoinTaskWorkUnit(query2);
            List<ProTask> proTasks1 = proTaskService.selectProTaskJoinTaskWorkUnit(query);
            for (ProTask task : proTasks1) {
                for (ProTask proTask1 : proTasks) {
                    if (task.getTaskId().equals(proTask1.getTaskId())) {
                        proTask1.setLocation(1L);
                    }
                }
            }
            list.addAll(proTasks);
        } else {
            list = proTaskService.selectProTaskJoinTaskWorkUnit(query);
        }
        rst = list.stream().filter(t -> !TaskStatusEnum.FINISHED.getStatus().equals(t.getStatus())).collect(Collectors.toList());
        if (CollectionUtil.isNotEmpty(rst)) {
            for (ProTask task : rst) {
                String arrangeCode = task.getArrangeCode();
                if (StringUtils.isNotEmpty(arrangeCode)) {
                    ProWorkorderQuery workorderQuery = new ProWorkorderQuery();
                    workorderQuery.setArrangeCode(arrangeCode);
                    List<ProWorkorder> proWorkorders = proWorkorderService.selectProWorkorderList(workorderQuery);
                    task.setProWorkorderList(proWorkorders);
                }
            }
        }
        return getDataTable(rst);
    }

    /**
     * 根据TaskCode获取生产任务详情
     *
     * @return
     */
    @GetMapping("/getProTaskByTaskCode")
    public AjaxResult getProTaskByTaskCode(@RequestParam("taskCode") String taskCode) {
        return AjaxResult.success(proTaskService.selectProTaskByTaskCode(taskCode));
    }

    /**
     * 获取生产任务详细信息
     */
    @ApiOperation("查询生产任务详情接口")
    @PreAuthorize("@ss.hasPermi('mes:pro:protask:query')")
    @GetMapping(value = "/{taskId}")
    public AjaxResult getInfo(@PathVariable("taskId") Long taskId) {
        return AjaxResult.success(proTaskService.selectProTaskByTaskId(taskId));
    }


    /**
     * 修改生产任务状态
     */
    @ApiOperation("修改生产任务状态接口")
    @Log(title = "生产任务", businessType = BusinessType.UPDATE)
    @PostMapping("/change")
    @ResponseBody
    public AjaxResult changeStatus(ProTask proTask) {
        proTaskService.changeStatus(proTask);
        return AjaxResult.success();
    }


    @ApiOperation("生产报工")
    @Log(title = "生产报工", businessType = BusinessType.INSERT)
    @PostMapping("/feedback")
    @ResponseBody
    public AjaxResult<List<ProPackagePrintResult>> feedBack(ProFeedback feedback) {
        return AjaxResult.success(proTaskService.feedBack(feedback));
    }


    @ApiOperation("包装工序生产报工")
    @Log(title = "包装工序生产报工", businessType = BusinessType.INSERT)
    @PostMapping("/feedBackPackage")
    @ResponseBody
    public AjaxResult<List<WorkorderPackagePrint>> feedBackPackage(FeedbackPackageAddParams feedback) {
        return AjaxResult.success(proTaskService.feedBackPackage(feedback));
    }

    @ApiOperation("包装工序报工列表")
    @PostMapping("/listFeedBackPackage")
    @ResponseBody
    public AjaxResult<FeedbackPackageInfo> listFeedBackPackage(@RequestBody FeedbackPackageQueryParams query) {
        if (query.getMergePack() == 1 && (query.getPackNum() == null || query.getPackNum() == 0)) {
            query.setPackNum(Integer.MAX_VALUE);
        }
        FeedbackPackageInfo rst = proTaskService.listFeedBackPackage(query);
        return AjaxResult.success(rst);
    }

    /**
     * pad端查看图片按钮显示的工单
     *
     * @return
     */
    @GetMapping("/getProWorkder")
    public AjaxResult getProWorkderTaskID(@RequestParam("taskId") Long taskId) {
        List<ProWorkorder> workorderList = new ArrayList<>();
        ProTaskWorkorder proTaskWorkorder = new ProTaskWorkorder();
        proTaskWorkorder.setTaskId(taskId);
        List<ProTaskWorkorder> proTaskWorkorders = proTaskWorkorderMapper.selectProTaskWorkorderList(proTaskWorkorder);
        for (ProTaskWorkorder pro : proTaskWorkorders) {
            QueryWrapper<ProWorkorder> proWorkorderQueryWrapper = new QueryWrapper<>();
            proWorkorderQueryWrapper.eq("workorder_id", pro.getWorkorderId());
            ProWorkorder proWorkorder = proWorkorderMapper.selectListByQw(proWorkorderQueryWrapper).get(0);
            workorderList.add(proWorkorder);
        }
        List<ProWorkorder> collect = workorderList.stream().collect(
                collectingAndThen(
                        toCollection(() -> new TreeSet<>(Comparator.comparing(ProWorkorder::getProductName))),
                        ArrayList::new));

        return AjaxResult.success(collect);
    }

    /**
     * 查看下一道工序
     */
    @GetMapping("/feedBackNextProcess")
    public AjaxResult feedbackNextProcess(ProTask protask) {
        ProTask task = proTaskService.selectProTaskByTaskId(protask.getTaskId());
        ProTask task1 = new ProTask();
        QueryWrapper<ProTask> query = new QueryWrapper<>();
        query.notIn("task_id", protask.getTaskId());
        query.eq("task_batch", task.getTaskBatch());
        query.ge("ordinal", task.getOrdinal());
        query.orderByAsc("ordinal");
        List<ProTask> proTasks = proTaskMapper.selectProTaskJoinProcess(query);
        if (CollUtil.isNotEmpty(proTasks)) {
            task1 = proTasks.get(0);
            if (Objects.nonNull(task1)) {
                ProTask proTask = proTaskMapper.selectWorkunitTaskId(task1.getTaskId());
                task1.setWorkunitName(proTask.getWorkunitName());
            }
        }
        return AjaxResult.success(task1);
    }


}
