package com.bsn.mes.pro.mapper.proWorkOrder;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderProcess;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderProcessItem;
import com.bsn.mes.pro.domain.vo.ProWorkOrderProcessVo;
import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.Select;

import java.util.List;

/**
 * 生产工单工序Mapper接口
 *
 * @author yinjinlu
 * @date 2024-01-23
 */
public interface ProWorkOrderProcessMapper {
    /**
     * 根据工单id查询所属所有工序
     *
     * @param workorderId
     * @return
     */
    List<ProWorkOrderProcessVo> selectProWorkOrderProcessByWorkOrderId(Long workorderId);


    @Select("select idx from pro_workorder pw left join mes.pro_work_order_process pwop on pw.workorder_id = pwop.workorder_id where pw.workorder_code = (\n" +
            "select workorder_code from pro_workorder where arrange_code = #{arrangeCode} limit 1) and pwop.process_id = #{processId}\n")
    String selectProcessIndexByObj(@Param("processId") String processId, @Param("arrangeCode") Long arrangeCode);


    /**
     * 查询所属所有工序
     *
     * @param query
     * @return
     */
    @Select(value = "select pwop.workorder_process_id,\n" +
            "               pwop.workorder_id,\n" +
            "               pwop.workstation_id,\n" +
            "               pwop.std_working_time,\n" +
            "               pwop.default_pre_time,\n" +
            "               pwop.default_suf_time,\n" +
            "               pwop.key_flag,\n" +
            "               pwop.is_check,\n" +
            "               pwop.remark,\n" +
            "               pp.process_name,\n" +
            "               pp.process_code,\n" +
            "               pp.process_id,\n" +
            "               pwop.idx,\n" +
            "               mw.workstation_name\n" +
            "        from pro_work_order_process pwop\n" +
            "                 left join pro_process pp on pwop.process_id = pp.process_id\n" +
            "                 left join md_workstation mw on pwop.workstation_id = mw.workstation_id\n" +
            "${ew.customSqlSegment} order by pwop.idx")
    List<ProWorkOrderProcessVo> selectProWorkOrderProcess(@Param("ew") QueryWrapper<ProWorkOrderProcessVo> query);

    /**
     * 查询生产工单工序
     *
     * @param workorderProcessId 生产工单工序主键
     * @return 生产工单工序
     */
    ProWorkOrderProcessVo selectProWorkOrderProcessByWorkorderProcessId(Long workorderProcessId);

    /**
     * 查询生产工单工序列表
     *
     * @param proWorkOrderProcess 生产工单工序
     * @return 生产工单工序集合
     */
    List<ProWorkOrderProcessVo> selectProWorkOrderProcessList(ProWorkOrderProcess proWorkOrderProcess);

    /**
     * 新增生产工单工序
     *
     * @param proWorkOrderProcess 生产工单工序
     * @return 结果
     */
    int insertProWorkOrderProcess(ProWorkOrderProcess proWorkOrderProcess);

    /**
     * 修改生产工单工序
     *
     * @param proWorkOrderProcess 生产工单工序
     * @return 结果
     */
    int updateProWorkOrderProcess(ProWorkOrderProcess proWorkOrderProcess);

    /**
     * 删除生产工单工序
     *
     * @param workorderProcessId 生产工单工序主键
     * @return 结果
     */
    int deleteProWorkOrderProcessByWorkorderProcessId(Long workorderProcessId);

    /**
     * 批量删除生产工单工序
     *
     * @param workorderProcessIds 需要删除的数据主键集合
     * @return 结果
     */
    int deleteProWorkOrderProcessByWorkorderProcessIds(Long[] workorderProcessIds);


    @Select(value = "select t1.*,t2.workstation_name,t2.workstation_code,t3.product_id,t3.product_name,t3.arrange_code from pro_work_order_process t1 left join md_workstation t2 on t1.workstation_id = t2.workstation_id left join pro_workorder t3 on t1.workorder_id = t3.workorder_id\n" +
            "${ew.customSqlSegment}")
    List<ProWorkOrderProcess> selectListByQw(@Param("ew") QueryWrapper<ProWorkOrderProcess> queryProcess);

    /**
     * 删除生产工单工序
     *
     * @param workorderId 生产工单主键
     * @return 结果
     */
    int deleteProWorkOrderProcessByWorkorderId(Long workorderId);


    @Delete("delete from pro_work_order_process where workorder_code=#{workorderCode} ")
    void deleteByWorkorderCode(@Param("workorderCode") String workorderCode);

    @Select("select  DISTINCT w.requirement_number,w.requirement_project_number ,md.item_id, md.item_code, md.item_name, md.specification, md.item_type_name\n" +
            "           t, md.unit_of_measure\n" +
            "            from pro_work_order_process_item w\n" +
            "            left join pro_work_order_process m on w.workorder_process_id = m.workorder_process_id\n" +
            "            left join md_item md on w.item_id = md.item_id  ${ew.customSqlSegment}")
    List<ProWorkOrderProcessItem> selectProWorkOrderProcessOrWorkorder(@Param("ew") QueryWrapper<ProWorkOrderProcess> queryProcess);

    @Delete(value = "delete from pro_work_order_process  ${ew.customSqlSegment}")
    void deleteProWorkOrderProcess(@Param("ew") QueryWrapper<ProWorkOrderProcess> queryWrapper);
}
