package com.bsn.mes.pro.schedule;

import lombok.Data;

import java.math.BigDecimal;
import java.time.Duration;
import java.util.*;

@Data
public class Task {

    public Task(Long id, Job job, List<Equipment> optionalEquipments, IOperationTimeCalculator operationTimeCalculator, ILimitedTimeCalculator limitedTimeCalculator)
    {
        this.id = id;
        this.job = job;
        this.optionalEquipments = optionalEquipments;
        this.operationTimeCalculator = operationTimeCalculator;
        this.limitedTimeCalculator = limitedTimeCalculator;
        taskData = new HashMap<String, Object>();

        this.lmtStartedTime = new OptionalTimeSpan(OptionalTimeSpan.Type.Max);
        this.lmtEndedTime = new OptionalTimeSpan(OptionalTimeSpan.Type.Min);

        this.lmtStartedTime.addOptional(() -> job.getLmtStartedTime().get());
        this.lmtEndedTime.addOptional(() -> job.getLmtEndedTime().get());

        scheduleStatus = ScheduleStatus.UnScheduled;
    }

    public Task(Long id, Job job)
    {
        this.id = id;
        this.job = job;
        taskData = new HashMap<String, Object>();

        this.lmtStartedTime = new OptionalTimeSpan(OptionalTimeSpan.Type.Max);
        this.lmtEndedTime = new OptionalTimeSpan(OptionalTimeSpan.Type.Min);

        this.lmtStartedTime.addOptional(() -> job.getLmtStartedTime().get());
        this.lmtEndedTime.addOptional(() -> job.getLmtEndedTime().get());

        scheduleStatus = ScheduleStatus.UnScheduled;
    }

    /*
     * Id
     */
    private Long id;

    /*
     * Job
     */
    private Job job;

    private Long processId;

    /**
     * 工序名
     */
    private String processName;

    /**
     * 工作中心名称
     */
    private Long workCenterId;

    /**
     * 工作中心名称
     */
    private String workCenterName;

    /**
     * 标准工时
     */
    private Duration standardWorkingTime;

    /**
     * 换型时间
     */
    private Duration setupTime=Duration.ZERO;

    /**
     * 订单数量
     */
    private BigDecimal orderQuantity;

    /**
     * 是否外委工序
     *
     */
    private boolean outsourced=false;

    /*
     * Task Data
     */
    private Map<String, Object> taskData;

    /*
     * 可选设备
     */
    private List<Equipment> optionalEquipments;

    /*
     * 排产设备
     */
    private Equipment scheduledEquipment;

    /*
     * 工作时间计算器
     */
    private IOperationTimeCalculator operationTimeCalculator;

    /*
     * 开始结束时间限制线计算器
     */
    private ILimitedTimeCalculator limitedTimeCalculator;

    /*
     * 开始时间限制线
     */
    private OptionalTimeSpan lmtStartedTime;

    /*
     * 结束时间限制线
     */
    private OptionalTimeSpan lmtEndedTime;

    /*
     * 排产开始时间
     */
    private Duration scheduledStartedTime;

    /*
     * 排产结束时间
     */
    private Duration scheduledEndedTime;

    /*
     * 排产结果
     */
    private ScheduleStatus scheduleStatus;

    /*
     * 作业日历
     */
    private List<Calendar.OccupiedCalendar> occupiedCalendar;

    private CalendarManager calendarManager;

    /*
     * 按设备拆分后的任务
     */
    List<Task> splitTask;

    public List<Calendar.OccupiedCalendar> getOccupiedCalendar() {
        return calendarManager.getOccupiedCalendar(this);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Task task = (Task) o;
        return Objects.equals(id, task.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }
}
