package com.bsn.mes.pro.schedule.strategy;

import com.bsn.mes.pro.schedule.IOperationTimeCalculator;
import com.bsn.mes.pro.schedule.TaskSchedulingContext;

import java.time.Duration;
import java.util.List;

/**
 * 交货时间评估设备
 * 1. 最后时间是否在订单要求时间范围内
 *    在交期范围内，3
 *    在交期范围外，-1
 * 2。最早完成任务 0-1（浮点数），1/最后完成分钟
 *
 */
public class EvaluateEquipmentDeliveryTime implements EvaluateEquipment{
    @Override
    public double evaluate(TaskSchedulingContext context, IOperationTimeCalculator.OperationTimePlan operationTimePlan) {
        double rst = 0;
        //最少换型
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItemList = operationTimePlan.getPlan();
        if(operationTimePlanItemList.isEmpty()){
            return rst;
        }
        Duration lastEndTime = operationTimePlanItemList.get(operationTimePlanItemList.size()-1).getEndedTime();
        //完成时间评分
        double finishDateScore=1.0/lastEndTime.toMinutes();
        rst+=finishDateScore;
        // 在交期范围内，3
        // 在交期范围外，-1
        if(context.getJob().getProducedDeliveryTime().compareTo(lastEndTime)>=0){
            rst += 3;
        }else{
            rst += -1;
        }
        return rst;
    }

}
