package com.bsn.mes.pro.service.impl.proWorkOrder;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.bsn.common.constant.HttpStatus;
import com.bsn.mes.ep.EpService;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderChangeRecord;
import com.bsn.mes.pro.domain.sap.SAPDtoProWorkOrder;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkOrderChangeRecordMapper;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderChangeRecordService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 工单变更记录Service业务层处理
 *
 * @author generator
 * @date 2024-04-18
 */
@Service
public class ProWorkOrderChangeRecordServiceImpl implements IProWorkOrderChangeRecordService {
    @Autowired
    private ProWorkOrderChangeRecordMapper proWorkOrderChangeRecordMapper;



    @Autowired
    private EpService epService;

    /**
     * 查询工单变更记录
     *
     * @param id 工单变更记录主键
     * @return 工单变更记录
     */
    @Override
    public ProWorkOrderChangeRecord selectProWorkOrderChangeRecordById(Long id) {
        return proWorkOrderChangeRecordMapper.selectProWorkOrderChangeRecordById(id);
    }

    /**
     * 查询工单变更记录列表
     *
     * @param proWorkOrderChangeRecord 工单变更记录
     * @return 工单变更记录
     */
    @Override
    public List<ProWorkOrderChangeRecord> selectProWorkOrderChangeRecordList(ProWorkOrderChangeRecord proWorkOrderChangeRecord) {
        return proWorkOrderChangeRecordMapper.selectProWorkOrderChangeRecordList(proWorkOrderChangeRecord);
    }

    /**
     * 新增工单变更记录
     *
     * @param proWorkOrderChangeRecord 工单变更记录
     * @return 结果
     */
    @Override
    public int insertProWorkOrderChangeRecord(ProWorkOrderChangeRecord proWorkOrderChangeRecord) {
        proWorkOrderChangeRecord.createAction();
        return proWorkOrderChangeRecordMapper.insertProWorkOrderChangeRecord(proWorkOrderChangeRecord);
    }

    /**
     * 修改工单变更记录
     *
     * @param proWorkOrderChangeRecord 工单变更记录
     * @return 结果
     */
    @Override
    public int updateProWorkOrderChangeRecord(ProWorkOrderChangeRecord proWorkOrderChangeRecord) {
        proWorkOrderChangeRecord.updateAction();
        return proWorkOrderChangeRecordMapper.updateProWorkOrderChangeRecord(proWorkOrderChangeRecord);
    }

    /**
     * 批量删除工单变更记录
     *
     * @param ids 需要删除的工单变更记录主键
     * @return 结果
     */
    @Override
    public int deleteProWorkOrderChangeRecordByIds(Long[] ids) {
        return proWorkOrderChangeRecordMapper.deleteProWorkOrderChangeRecordByIds(ids);
    }

    /**
     * 删除工单变更记录信息
     *
     * @param id 工单变更记录主键
     * @return 结果
     */
    @Override
    public int deleteProWorkOrderChangeRecordById(Long id) {
        return proWorkOrderChangeRecordMapper.deleteProWorkOrderChangeRecordById(id);
    }

    /**
     * 查询SAP平台流水号
     *
     * @param sapDtoProWorkOrder
     * @return
     */
    @Override
    public Integer querySAPSerial(SAPDtoProWorkOrder sapDtoProWorkOrder) {
        Integer currentSerial = null;
        //客户编码
        String clientCode = sapDtoProWorkOrder.getClientCode();
        //物料编号
        String sapItemCode = sapDtoProWorkOrder.getSapItemCode();
        //工单数量
        BigDecimal quantity = sapDtoProWorkOrder.getQuantity();
        Map<String, Object> map = new HashMap<>();
        map.put("clientNo", clientCode);
        map.put("itemNo", sapItemCode);
        String body = JSON.toJSONString(map);
        //先通过远程调用获取当前流水号
        String epItemSerial = epService.getEpItemSerial(body);
        //String epItemSerial = HttpUtils.sendPost("http://192.168.3.91:12000/api/itemSerial/get", body);
        JSONObject jsonObject = JSON.parseObject(epItemSerial);
        String code = jsonObject.getString("code");
        String statusCode=String.valueOf(HttpStatus.SUCCESS);
        if (code.equals(statusCode)) {
            JSONObject data = jsonObject.getJSONObject("data");
            if (data != null) {
                //当前流水码
                 currentSerial = data.getInteger("currentSerial");
                //重新生成当前流水码,生成规则: 新的当前流水码=获得的当前流水码+工单数量
                Integer newCurrentSerial = currentSerial + quantity.intValue();
                //将新的当前流水码传入EP系统
                Map<String, Object> newMap = new HashMap<>();
                newMap.put("clientNo", clientCode);
                newMap.put("itemNo", sapItemCode);
                newMap.put("currentSerial", newCurrentSerial);
                String bodys = JSON.toJSONString(newMap);
                String s =epService.updateEpItemSerial(bodys);
                //String s = HttpUtils.sendPost("http://192.168.3.91:12000/api/itemSerial/updCurrent", bodys);
                System.out.println(s);
            }
        }
        return currentSerial;
    }
}
