package com.bsn.mes.tm.controller;

import com.bsn.common.annotation.Log;
import com.bsn.common.core.controller.BaseController;
import com.bsn.common.core.domain.AjaxResult;
import com.bsn.common.core.page.TableDataInfo;
import com.bsn.common.enums.BusinessType;
import com.bsn.common.utils.poi.ExcelUtil;
import com.bsn.mes.tm.domain.TmToolRequest;
import com.bsn.mes.tm.service.ITmToolRequestService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 刀模版申请单Controller
 *
 * @author generator
 * @date 2024-02-02
 */
@RestController
@RequestMapping("/tm/tmToolRequest")
public class TmToolRequestController extends BaseController {
    @Autowired
    private ITmToolRequestService tmToolRequestService;

    /**
     * 查询刀模版申请单列表
     */
    @PreAuthorize("@ss.hasPermi('tm:tmToolRequest:list')")
    @GetMapping("/list")
    public TableDataInfo list(TmToolRequest tmToolRequest) {
        startPage();
        List<TmToolRequest> list = tmToolRequestService.selectTmToolRequestList(tmToolRequest);
        return getDataTable(list);
    }

    /**
     * 导出刀模版申请单列表
     */
    @PreAuthorize("@ss.hasPermi('tm:tmToolRequest:export')")
    @Log(title = "刀模版申请单", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, TmToolRequest tmToolRequest) {
        List<TmToolRequest> list = tmToolRequestService.selectTmToolRequestList(tmToolRequest);
        ExcelUtil<TmToolRequest> util = new ExcelUtil<TmToolRequest>(TmToolRequest.class);
        util.exportExcel(response, list, "刀模版申请单数据");
    }

    /**
     * 获取刀模版申请单详细信息
     */
//    @PreAuthorize("@ss.hasPermi('tm:tmToolRequest:query')")
    @GetMapping(value = "/{toolRequestId}")
    public AjaxResult getInfo(@PathVariable("toolRequestId") Long toolRequestId) {
        return AjaxResult.success(tmToolRequestService.selectTmToolRequestByToolRequestId(toolRequestId));
    }

    /**
     * 新增刀模版申请单
     */
    @PreAuthorize("@ss.hasPermi('tm:tmToolRequest:add')")
    @Log(title = "刀模版申请单", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody TmToolRequest tmToolRequest) {
        return toAjax(tmToolRequestService.insertTmToolRequestAndItem(tmToolRequest));
    }

    /**
     * 修改刀模版申请单
     */
    @PreAuthorize("@ss.hasPermi('tm:tmToolRequest:edit')")
    @Log(title = "刀模版申请单", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody TmToolRequest tmToolRequest) {
        return toAjax(tmToolRequestService.updateTmToolRequest(tmToolRequest));
    }

    /**
     * 删除刀模版申请单
     */
    @PreAuthorize("@ss.hasPermi('tm:tmToolRequest:remove')")
    @Log(title = "刀模版申请单", businessType = BusinessType.DELETE)
    @DeleteMapping("/{toolRequestIds}")
    public AjaxResult remove(@PathVariable Long[] toolRequestIds) {
        return toAjax(tmToolRequestService.deleteTmToolRequestByToolRequestIds(toolRequestIds));
    }
}
