package com.bsn.mes.tm.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.bsn.common.constant.UserConstants;
import com.bsn.common.utils.data.DataUtil;
import com.bsn.common.utils.data.ExceptionUtil;
import com.bsn.common.utils.data.StringUtils;
import com.bsn.mes.constant.TaskWorkunitStatusEnum;
import com.bsn.mes.constant.TmMachineCheckErrorEnum;
import com.bsn.mes.constant.TmMachineTypeEnum;
import com.bsn.mes.constant.ToolStatusEnum;
import com.bsn.mes.md.domain.MdWorkunit;
import com.bsn.mes.md.mapper.MdWorkunitMapper;
import com.bsn.mes.md.vo.MdItemVo;
import com.bsn.mes.md.vo.MdWorkunitVo;
import com.bsn.mes.pro.domain.ProArrange;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderProcess;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkOrderProcessTool;
import com.bsn.mes.pro.domain.proWorkOrder.ProWorkorder;
import com.bsn.mes.pro.domain.task.ProTask;
import com.bsn.mes.pro.domain.task.ProTaskWorkunit;
import com.bsn.mes.pro.mapper.ProArrangeMapper;
import com.bsn.mes.pro.mapper.proWorkOrder.ProWorkOrderProcessToolMapper;
import com.bsn.mes.pro.mapper.task.ProTaskMapper;
import com.bsn.mes.pro.mapper.task.ProTaskWorkunitMapper;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderProcessService;
import com.bsn.mes.pro.service.proWorkOrder.IProWorkOrderProcessToolService;
import com.bsn.mes.tm.domain.TmTool;
import com.bsn.mes.tm.domain.TmToolMachines;
import com.bsn.mes.tm.domain.TmToolRequestUseItem;
import com.bsn.mes.tm.mapper.TmToolMachinesMapper;
import com.bsn.mes.tm.mapper.TmToolMapper;
import com.bsn.mes.tm.mapper.TmToolRequestUseItemMapper;
import com.bsn.mes.tm.service.ITmToolMachinesService;
import com.bsn.system.strategy.AutoCodeUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 刀模版上下机Service业务层处理
 *
 * @author generator
 * @date 2024-02-06
 */
@Slf4j
@Service
public class TmToolMachinesServiceImpl implements ITmToolMachinesService {
    @Autowired
    private TmToolMachinesMapper tmToolMachinesMapper;


    @Autowired
    private TmToolMapper tmToolMapper;
    @Autowired
    private TmToolRequestUseItemMapper tmToolRequestUseItemMapper;

    @Autowired
    private AutoCodeUtil autoCodeUtil;

    @Autowired
    private IProWorkOrderProcessToolService proWorkOrderProcessToolService;
    @Autowired
    private ProWorkOrderProcessToolMapper proWorkOrderProcessToolMapper;

    @Autowired
    private IProWorkOrderProcessService proWorkOrderProcessService;

    @Autowired
    private ProArrangeMapper proArrangeMapper;

    @Autowired
    private ProTaskWorkunitMapper proTaskWorkunitMapper;

    @Autowired
    private ProTaskMapper proTaskMapper;


    @Autowired
    private MdWorkunitMapper mdWorkunitMapper;

    /**
     * 查询刀模版上下机
     *
     * @param toolMachinesId 刀模版上下机主键
     * @return 刀模版上下机
     */
    @Override
    public TmToolMachines selectTmToolMachinesByToolMachinesId(Long toolMachinesId) {
        return tmToolMachinesMapper.selectTmToolMachinesByToolMachinesId(toolMachinesId);
    }

    /**
     * 查询刀模版上下机列表
     *
     * @param tmToolMachines 刀模版上下机
     * @return 刀模版上下机
     */
    @Override
    public List<TmToolMachines> selectTmToolMachinesList(TmToolMachines tmToolMachines) {
        return tmToolMachinesMapper.selectTmToolMachinesList(tmToolMachines);
    }


    /**
     * 查询刀模版上下机列表
     *
     * @param tmToolMachines 刀模版上下机
     * @return 刀模版上下机
     */
    @Override
    public List<TmToolMachines> selectTmToolMachinesVoList(TmToolMachines tmToolMachines) {
        return tmToolMachinesMapper.selectTmToolMachinesVoList(tmToolMachines);
    }

    /**
     * 新增刀模版上下机
     *
     * @param tmToolMachinesList 刀模版上下机
     * @return 结果
     */
    @Override
    public int insertTmToolMachines(List<TmToolMachines> tmToolMachinesList) {
        int i = 0;
        if (CollectionUtil.isEmpty(tmToolMachinesList)) {
            log.info("传入的记录为空");
            return i;
        }

        for (TmToolMachines tmToolMachines : tmToolMachinesList) {

            // 编码如果为空则自动生成
            if (StringUtils.isEmpty(tmToolMachines.getToolMachinesCode())) {
                tmToolMachines.setToolMachinesCode(autoCodeUtil.genSerialCode(UserConstants.TOOL_MACHINES_CODE, null));
            }

            //新增主表数据
            i = i + this.insertTmToolMachines(tmToolMachines);
        }

        return i;
    }

    @Override
    public int insertTmToolMachines(TmToolMachines tmToolMachines) {
        tmToolMachines.createAction();
        return tmToolMachinesMapper.insertTmToolMachines(tmToolMachines);
    }

    /**
     * 修改刀模版上下机
     *
     * @param tmToolMachines 刀模版上下机
     * @return 结果
     */
    @Override
    public int updateTmToolMachines(TmToolMachines tmToolMachines) {
        tmToolMachines.updateAction();
        return tmToolMachinesMapper.updateTmToolMachines(tmToolMachines);
    }

    /**
     * 批量删除刀模版上下机
     *
     * @param toolMachinesIds 需要删除的刀模版上下机主键
     * @return 结果
     */
    @Override
    public int deleteTmToolMachinesByToolMachinesIds(Long[] toolMachinesIds) {
        return tmToolMachinesMapper.deleteTmToolMachinesByToolMachinesIds(toolMachinesIds);
    }

    /**
     * 删除刀模版上下机信息
     *
     * @param toolMachinesId 刀模版上下机主键
     * @return 结果
     */
    @Override
    public int deleteTmToolMachinesByToolMachinesId(Long toolMachinesId) {
        return tmToolMachinesMapper.deleteTmToolMachinesByToolMachinesId(toolMachinesId);
    }

    @Transactional
    @Override
    public int toolUpdateMountStatus(List<TmToolMachines> tmToolMachinesList) {
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmToolMachinesList), "入库记录为空");


        Integer action = tmToolMachinesList.get(0).getType();
        List<Long> machinesIds = tmToolMachinesList.stream().map(TmToolMachines::getToolMachinesId).collect(Collectors.toList());
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(machinesIds), "需要修改的记录为空");


        QueryWrapper<TmToolMachines> machinesQueryWrapper = new QueryWrapper<>();
        machinesQueryWrapper.in("t1.tool_machines_id", machinesIds);
        List<TmToolMachines> toolMachinesList = tmToolMachinesMapper.selectListByQw(machinesQueryWrapper);
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(toolMachinesList) || toolMachinesList.size() < machinesIds.size(), "该刀模版具不属于当前工作单元");
        Set<String> errorBondingSet = toolMachinesList.stream().filter(x -> x.getType() == null || x.getToolRequestUseItemId() == null || x.getToolRequestUseId() == null).map(x -> x.getToolMachinesId().toString()).collect(Collectors.toSet());
        ExceptionUtil.checkTruethrowException(CollectionUtil.isNotEmpty(errorBondingSet), "上机记录" + errorBondingSet + "尚未绑定刀模版具,不可进行手动上下机");
        Set<String> toolCodes = toolMachinesList.stream().map(TmToolMachines::getToolCode).collect(Collectors.toSet());
        checkTmTool(toolCodes);

        if (Objects.equals(action, TmMachineTypeEnum.MOUNT.getType())) {
            QueryWrapper<TmToolMachines> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("tt.tool_code", toolCodes);
            queryWrapper.in("t1.type", TmMachineTypeEnum.DIS_MOUNT.getType());
            List<TmToolMachines> tmToolMachinesList3 = tmToolMachinesMapper.selectListByQw(queryWrapper);
            ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmToolMachinesList3), "sn:" + tmToolMachinesList3.stream().map(TmToolMachines::getToolCode).collect(Collectors.toList()) + ",无记录可上机");
        }

        if (Objects.equals(action, TmMachineTypeEnum.DIS_MOUNT.getType())) {
            QueryWrapper<TmToolMachines> queryWrapper = new QueryWrapper<>();
            queryWrapper.in("tt.tool_code", toolCodes);
            queryWrapper.in("t1.task_workunit_id", toolMachinesList.stream().map(TmToolMachines::getTaskWorkunitId).filter(Objects::nonNull).collect(Collectors.toSet()));
            queryWrapper.in("t1.type", TmMachineTypeEnum.MOUNT.getType());
            List<TmToolMachines> tmToolMachines = tmToolMachinesMapper.selectListByQw(queryWrapper);
            ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmToolMachines), "sn:" + toolCodes + ",无记录可下机");
        }

        Map<Long, TmToolMachines> tmToolMachinesMap = toolMachinesList.stream().collect(Collectors.toMap(TmToolMachines::getToolMachinesId, x -> x));
        Map<Long, TmToolMachines> tmToolMachinesArgsMap = tmToolMachinesList.stream().collect(Collectors.toMap(TmToolMachines::getToolMachinesId, x -> x));

        for (Long machinesId : machinesIds) {
            TmToolMachines tmToolMachines1 = tmToolMachinesMap.get(machinesId);
            TmToolMachines tmToolMachines2 = tmToolMachinesArgsMap.get(machinesId);
            if (Objects.equals(tmToolMachines1.getType(), tmToolMachines2.getType())) {
                ExceptionUtil.checkTruethrowException(Objects.equals(tmToolMachines1.getType(), TmMachineTypeEnum.DIS_MOUNT.getType()), "sn: " + tmToolMachines1.getToolCode() + ",该刀模版具已下机");
                ExceptionUtil.checkTruethrowException(Objects.equals(tmToolMachines1.getType(), TmMachineTypeEnum.MOUNT.getType()), "sn: " + tmToolMachines1.getToolCode() + ",该刀模版具已上机");
            }
        }

        int i = 0;
        for (TmToolMachines tmToolMachines : tmToolMachinesList) {
            List<TmToolMachines> toolMachines = tmToolMachinesMapper.selectListByQw(new QueryWrapper<TmToolMachines>().eq("tool_machines_id", tmToolMachines.getToolMachinesId()));
            TmToolMachines updated = toolMachines.get(0);
            if (null != action) {
                //上机修改状态，上机时间改为最新；下机修改状态，下机时间改为最新
                if (Objects.equals(TmMachineTypeEnum.MOUNT.getType(), action)) {
                    updated.setType(TmMachineTypeEnum.MOUNT.getType());
                    updated.setUpDate(new Date());
                } else if (Objects.equals(TmMachineTypeEnum.DIS_MOUNT.getType(), action)) {
                    updated.setType(TmMachineTypeEnum.DIS_MOUNT.getType());
                    updated.setDownDate(new Date());
                }
            }
            this.updateTmToolMachines(updated);
            i++;
        }
        return i;
    }

    @Override
    public List<TmToolMachines> selectListByQw(QueryWrapper<TmToolMachines> queryWrapper) {
        return tmToolMachinesMapper.selectListByQw(queryWrapper);
    }

    @Override
    public List<TmToolMachines> selectListJoinUseItemByQw(QueryWrapper<TmToolMachines> queryWrapper) {
        return tmToolMachinesMapper.selectListJoinUseItemByQw(queryWrapper);
    }

    // 报工满足条件下架刀模版具

    @Override
    public void feedbackToolDismount(ProTaskWorkunit taskWorkunit) {
        // 报工数量相等 自动下机
        if (taskWorkunit.getQuantityProduced().compareTo(taskWorkunit.getQuantity()) == 0) {
            QueryWrapper<TmToolMachines> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("type", TmMachineTypeEnum.MOUNT.getType());
            queryWrapper.eq("task_workunit_id", taskWorkunit.getTaskWorkunitId());
            List<TmToolMachines> tmToolMachinesList = this.selectListByQw(queryWrapper);

            for (TmToolMachines tmToolMachines : tmToolMachinesList) {
                tmToolMachines.setType(TmMachineTypeEnum.DIS_MOUNT.getType());
                this.updateTmToolMachines(tmToolMachines);
            }
        }

    }


    // 工单完结下架刀模版具
    @Override
    public void taskComplateToolDismount(ProTaskWorkunit proTaskWorkunit) {
        if (Objects.equals(proTaskWorkunit.getStatus(), TaskWorkunitStatusEnum.FINISHED.getStatus())) {
            QueryWrapper<TmToolMachines> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("type", TmMachineTypeEnum.MOUNT.getType());
            queryWrapper.eq("task_workunit_id", proTaskWorkunit.getTaskWorkunitId());
            List<TmToolMachines> tmToolMachinesList = this.selectListByQw(queryWrapper);
            for (TmToolMachines tmToolMachines : tmToolMachinesList) {
                tmToolMachines.setType(TmMachineTypeEnum.DIS_MOUNT.getType());
                this.updateTmToolMachines(tmToolMachines);
            }
        }
    }


    // 检查刀模板具
    @Override
    public int checkWhetherTheToolTemplateIsSatisfied(Long taskWorkunitId) {
        // 检查是否需要绑定刀模版具
        Map<Long, Integer> toolNeedMap = proWorkOrderProcessToolService.selectRequiredToolForWorkunitTask(taskWorkunitId);
        if (CollectionUtil.isEmpty(toolNeedMap)) {
            // 无需绑定刀模板具
            return TmMachineCheckErrorEnum.ERROR_06.getStatus();
        }


        QueryWrapper<TmToolMachines> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("t1.task_workunit_id", taskWorkunitId);
        List<TmToolMachines> tmToolMachinesList = tmToolMachinesMapper.selectListByQw(queryWrapper1);

        // 刀模板记录未创建
        if (CollectionUtil.isEmpty(tmToolMachinesList)) {
            return TmMachineCheckErrorEnum.ERROR_01.getStatus();
        }

        // 创建的记录不够
        IntSummaryStatistics collect = toolNeedMap.values().stream().filter(Objects::nonNull).collect(Collectors.summarizingInt(x -> x));
        if ((tmToolMachinesList.size() < collect.getSum())) {
            return TmMachineCheckErrorEnum.ERROR_01.getStatus();
        }


        // 已经绑定的数据
        List<TmToolMachines> machinesBondingList = tmToolMachinesList.stream().filter(x -> x.getToolRequestUseItemId() != null).filter(x -> x.getItemId() != null).collect(Collectors.toList());

        // 已经绑定刀具的记录为空
        Map<Long, List<TmToolMachines>> tmtoolBondingMap = machinesBondingList.stream().collect(Collectors.groupingBy(TmToolMachines::getItemId));
        if (CollectionUtil.isEmpty(tmtoolBondingMap)) {
            return TmMachineCheckErrorEnum.ERROR_04.getStatus();
        }

        // 已绑定数刀模板记录不够小于必须需要的刀具
        for (Map.Entry<Long, List<TmToolMachines>> entry : tmtoolBondingMap.entrySet()) {
            List<TmToolMachines> value = entry.getValue();
            Integer i = DataUtil.getNormalData(toolNeedMap.get(entry.getKey()));
            if (value.size() < i) {
                return TmMachineCheckErrorEnum.ERROR_04.getStatus();
            }
        }


        // 已上机的刀具
        Map<Long, List<TmToolMachines>> tmtoolMountMap = machinesBondingList.stream()
                .filter(x -> Objects.equals(x.getType(), TmMachineTypeEnum.MOUNT.getType()))
                .collect(Collectors.groupingBy(TmToolMachines::getItemId));
        if (CollectionUtil.isEmpty(tmtoolBondingMap)) {
            return TmMachineCheckErrorEnum.ERROR_04.getStatus();
        }


        Map<Long, List<TmToolMachines>> tmToolMachinesMap = tmToolMachinesList.stream().collect(Collectors.groupingBy(TmToolMachines::getItemId));
        for (Map.Entry<Long, List<TmToolMachines>> entry : tmToolMachinesMap.entrySet()) {
            Long key = entry.getKey();
            List<TmToolMachines> value = entry.getValue();
            List<TmToolMachines> machinesList = tmtoolMountMap.get(key);
            if (CollectionUtil.isEmpty(machinesList) || value.size() != machinesList.size()) {
                return TmMachineCheckErrorEnum.ERROR_04.getStatus();
            }
        }
        // 创建了并且全部上机
        return 0;
    }


    @Override
    public List<MdItemVo> selectUpTmToolRequestList(MdWorkunitVo mdWorkunit) {
        mdWorkunit.setType(1);
        return tmToolMachinesMapper.selectUpTmToolRequestList(mdWorkunit);
    }

    @Override
    public List<TmToolMachines> getRecordsByProcessIdAndArrangeCode(TmToolMachines tmToolMachines) {
        return tmToolMachinesMapper.selectTmToolMachinesVoList(tmToolMachines);
    }

    @Override
    public List<TmToolMachines> addObj(Long taskWorkunitId) {
        List<ProTaskWorkunit> proTaskWorkunits = proTaskWorkunitMapper.selectListByQw(new QueryWrapper<ProTaskWorkunit>().eq("task_workunit_id", taskWorkunitId));
        ExceptionUtil.checkTruethrowException(proTaskWorkunits.size() > 1, "排产任务查询错误");


        int b = this.checkWhetherTheToolTemplateIsSatisfied(taskWorkunitId);
        if (b != TmMachineCheckErrorEnum.ERROR_01.getStatus()) {
            return new ArrayList<>();
        }


        ProWorkorder proWorkorder = proTaskWorkunitMapper.selectProWorkorderByTaskWorkunitId(taskWorkunitId);
        ProTaskWorkunit proTaskWorkunit = proTaskWorkunits.get(0);
        QueryWrapper<ProTask> taskQueryWrapper = new QueryWrapper<>();
        taskQueryWrapper.eq("t1.task_id", proTaskWorkunit.getTaskId());
        List<ProTask> proTasks = proTaskMapper.selectListJoinByQw(taskQueryWrapper);
        ProTask proTask = proTasks.get(0);

        QueryWrapper<MdWorkunit> workunitQueryWrapper = new QueryWrapper<>();
        workunitQueryWrapper.eq("workunit_id", proTaskWorkunit.getWorkunitId());
        List<MdWorkunit> mdWorkunits = mdWorkunitMapper.selectListByQw(workunitQueryWrapper);
        MdWorkunit mdWorkunit = mdWorkunits.get(0);
//
//        QueryWrapper<TmToolRequest> requestQueryWrapper = new QueryWrapper<>();
//        requestQueryWrapper.eq("task_workunit_id", taskWorkunitId);
//        List<TmToolRequest> tmToolRequests = tmToolRequestMapper.selectListByQw(requestQueryWrapper);
//        TmToolRequest tmToolRequest = tmToolRequests.get(0);

        String arrangeCode = proWorkorder.getArrangeCode();
        QueryWrapper<ProArrange> arrangeQueryWrapper = new QueryWrapper<>();
        arrangeQueryWrapper.eq("arrange_code", arrangeCode);
        List<ProArrange> proArranges = proArrangeMapper.selectListByQw(arrangeQueryWrapper);
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(proArranges), "通过工单编排单号,没有查询到对应编排记录");
        Set<Long> workorderIds = proArranges.stream().map(ProArrange::getWorkorderId).collect(Collectors.toSet());

        QueryWrapper<ProWorkOrderProcessTool> processToolQueryWrapper = new QueryWrapper<>();
        processToolQueryWrapper.in("t1.work_order_id", workorderIds);
        processToolQueryWrapper.in("t3.process_id", proTask.getProcessId());
        List<ProWorkOrderProcessTool> processTools = proWorkOrderProcessToolMapper.selectListByQw(processToolQueryWrapper);
        if (CollectionUtil.isEmpty(processTools)) {
            return new ArrayList<>();
        }

        Map<Long, Optional<ProWorkOrderProcessTool>> processToolls = processTools.stream().filter(x -> x.getQuantity() != null
                && x.getQuantity().doubleValue() > 0).collect(Collectors.groupingBy(ProWorkOrderProcessTool::getToolId, Collectors.maxBy(Comparator.comparingDouble(x -> x.getQuantity().doubleValue()))));

        List<TmToolMachines> tmToolMachinesList = new ArrayList<>();

        Set<Long> workorderProcessIds = processToolls.values().stream().filter(x -> x.get() != null).map(x -> x.get().getWorkorderProcessId()).collect(Collectors.toSet());
        QueryWrapper<ProWorkOrderProcess> processQueryWrapper = new QueryWrapper<>();
        processQueryWrapper.in("t1.workorder_process_id", workorderProcessIds);
        List<ProWorkOrderProcess> proWorkOrderProcesses = proWorkOrderProcessService.selectListByQw(processQueryWrapper);
        Map<Long, Long> processMap = proWorkOrderProcesses.stream().collect(Collectors.toMap(ProWorkOrderProcess::getWorkorderProcessId, ProWorkOrderProcess::getProcessId));
        for (Map.Entry<Long, Optional<ProWorkOrderProcessTool>> entrySet : processToolls.entrySet()) {
            Optional<ProWorkOrderProcessTool> value = entrySet.getValue();
            if (value.isPresent()) {
                ProWorkOrderProcessTool proWorkOrderProcessTool = value.get();
                BigDecimal quantity = proWorkOrderProcessTool.getQuantity();
                Long processId = processMap.get(proWorkOrderProcessTool.getWorkorderProcessId());
                long l = quantity.longValue();
                for (long l1 = 0; l1 < l; l1++) {
                    TmToolMachines tmToolMachines = new TmToolMachines();
                    //开始封装数据进行新增
                    tmToolMachines.setItemId(proWorkOrderProcessTool.getToolId());
                    tmToolMachines.setWorkunitId(proTaskWorkunit.getWorkunitId());
                    tmToolMachines.setWorkunitName(mdWorkunit.getWorkunitName());
                    tmToolMachines.setWorkunitCode(mdWorkunit.getWorkunitCode());
                    tmToolMachines.setTaskId(proTaskWorkunit.getTaskId());
                    tmToolMachines.setTaskCode(proTask.getTaskCode());
                    tmToolMachines.setTaskName(proTask.getTaskName());
                    tmToolMachines.setTaskWorkunitId(taskWorkunitId);
                    tmToolMachines.setArrangeCode(arrangeCode);
                    tmToolMachines.setProcessId(processId);
                    tmToolMachines.setWorkorderId(proWorkOrderProcessTool.getWorkOrderId());
                    tmToolMachines.setUpDate(null);//默认当前上机时间
                    tmToolMachines.setDownDate(null);
                    tmToolMachinesList.add(tmToolMachines);
                }
            }
        }
        return tmToolMachinesList;
    }

    @Override
    public void addObj(TmToolMachines tmToolMachines) {
        this.insertTmToolMachines(addObj(tmToolMachines.getTaskWorkunitId()));
    }

    @Override
    public Integer bindTool(String toolCode, Integer type, TmToolMachines tmToolMachines) {
        QueryWrapper<TmTool> tmToolQueryWrapper = new QueryWrapper<>();
        tmToolQueryWrapper.eq("tool_code", toolCode);
        List<TmTool> tmTools = tmToolMapper.selectListByQw(tmToolQueryWrapper);
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmTools), "该台账未在系统录入");

        QueryWrapper<TmToolRequestUseItem> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("t2.tool_code", toolCode);
        List<TmToolRequestUseItem> tmToolRequestUseItems = tmToolRequestUseItemMapper.selecListByQw(queryWrapper);
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmToolRequestUseItems), "该工具未被领料");
        TmToolRequestUseItem tmToolRequestUseItem = tmToolRequestUseItems.get(0);
        return bindTool(tmToolMachines, tmToolRequestUseItem, type);
    }

    @Override
    public int bindTool(TmToolMachines tmToolMachines, TmToolRequestUseItem tmToolRequestUseItem, Integer type) {
        int count = 0;
        if (null != type) {
            //上机修改状态，上机时间改为最新；下机修改状态，下机时间改为最新
            if (Objects.equals(TmMachineTypeEnum.MOUNT.getType(), type)) {
                tmToolMachines.setType(TmMachineTypeEnum.MOUNT.getType());
                tmToolMachines.setUpDate(new Date());
            } else if (Objects.equals(TmMachineTypeEnum.DIS_MOUNT.getType(), type)) {
                tmToolMachines.setType(TmMachineTypeEnum.DIS_MOUNT.getType());
                tmToolMachines.setDownDate(new Date());
            } else if (Objects.equals(TmMachineTypeEnum.INIT.getType(), type)) {
                tmToolMachines.setType(TmMachineTypeEnum.INIT.getType());
                tmToolMachines.setUpDate(new Date());
                tmToolMachines.setDownDate(null);
            }
            tmToolMachines.setToolRequestUseId(tmToolRequestUseItem.getToolRequestUseId());
            tmToolMachines.setToolRequestUseItemId(tmToolRequestUseItem.getToolRequestUseItemId());
            count = this.updateTmToolMachines(tmToolMachines);
        }
        return count;

    }

    private void checkTmTool(Set<String> toolCodes) {
        List<TmTool> tmTools = tmToolMapper.selectListByQw(new QueryWrapper<TmTool>().in("tool_code", toolCodes));
        ExceptionUtil.checkTruethrowException(toolCodes.size() != tmTools.size(), "刀模板具未同步不可操作");
        for (TmTool tmTool : tmTools) {
            ExceptionUtil.checkTruethrowException(Objects.equals(tmTool.getStatus(), ToolStatusEnum.SCRAP.getStatus()) || Objects.equals(tmTool.getStatus(), ToolStatusEnum.STORE.getStatus()), "刀模板具未领料不可操作");
        }
    }

    @Override
    public Integer scan(TmToolMachines machines) {
        String sn = machines.getSn();
        ExceptionUtil.checkTruethrowException(sn == null || "".equals(sn) || !sn.contains("-"), "sn码非法,请检查");
        String[] snList = sn.split("-");
        String toolCode = snList[1];
        String sapItemCode = snList[0];
        checkTmTool(new HashSet<>(Collections.singletonList(toolCode)));

        Long taskWorkunitId = machines.getTaskWorkunitId();
        Integer type = machines.getType();

        List<TmToolMachines> tmToolMachines = new ArrayList<>();

        // 已经在使用中的tool_code
        QueryWrapper<TmToolMachines> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("tt.tool_code", toolCode);
        queryWrapper.eq("mi.sap_item_code", sapItemCode);
        List<TmToolMachines> tmToolMachinesList = tmToolMachinesMapper.selectListByQw(queryWrapper);
        if (Objects.equals(TmMachineTypeEnum.MOUNT.getType(), type)) {
            if (CollectionUtil.isNotEmpty(tmToolMachinesList)) {
                // 刀模版code没有找到上机记录
                boolean b = tmToolMachinesList.stream().anyMatch(x -> x.getToolRequestUseItemId() != null
                        && Objects.equals(TmMachineTypeEnum.MOUNT.getType(), x.getType()));
                ExceptionUtil.checkTruethrowException(b, "当前刀具已上机,请勿重复上机");

                // 查看本单元是否有上机记录,且不等于已下机
                List<TmToolMachines> collect = tmToolMachinesList.stream().filter(x -> x.getToolRequestUseItemId() != null
                        && Objects.equals(x.getTaskWorkunitId(), taskWorkunitId)
                        && Objects.equals(x.getToolCode(), toolCode)
                        && Objects.equals(TmMachineTypeEnum.DIS_MOUNT.getType(), x.getType())).collect(Collectors.toList());

                if (CollectionUtil.isNotEmpty(collect)) {
                    return bindTool(toolCode, type, collect.get(0));
                }
            }


        } else if (Objects.equals(TmMachineTypeEnum.DIS_MOUNT.getType(), type)) {
            ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmToolMachinesList), "当前刀具无可用下机记录");
            tmToolMachines = tmToolMachinesList.stream().filter(
                    x -> x.getToolRequestUseItemId() != null
                            && Objects.equals(TmMachineTypeEnum.MOUNT.getType(), x.getType())
                            && Objects.equals(x.getTaskWorkunitId(), taskWorkunitId)
            ).collect(Collectors.toList());
            ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmToolMachines), "当前刀具无可用下机记录");
            return bindTool(toolCode, type, tmToolMachines.get(0));
        }


        queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("t1.task_workunit_id", taskWorkunitId);
//        queryWrapper.eq("mi.sap_item_code", sapItemCode);
        List<TmToolMachines> toolMachines = tmToolMachinesMapper.selectListByQw(queryWrapper);
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(toolMachines), "没有匹配到对应的刀模具");
        tmToolMachines = toolMachines.stream().filter(x -> x.getToolRequestUseItemId() == null).collect(Collectors.toList());
        if (CollectionUtil.isEmpty(tmToolMachines)) {
            if (Objects.equals(TmMachineTypeEnum.MOUNT.getType(), type)) {
                tmToolMachines = toolMachines.stream().filter(
                        x -> x.getToolRequestUseItemId() != null
                                && !Objects.equals(TmMachineTypeEnum.MOUNT.getType(), x.getType())
                ).collect(Collectors.toList());
            } else if (Objects.equals(TmMachineTypeEnum.DIS_MOUNT.getType(), type)) {
                tmToolMachines = toolMachines.stream().filter(
                        x -> x.getToolRequestUseItemId() != null
                                && Objects.equals(TmMachineTypeEnum.MOUNT.getType(), x.getType())
                ).collect(Collectors.toList());
            }
        }
        ExceptionUtil.checkTruethrowException(CollectionUtil.isEmpty(tmToolMachines), "没有匹配到对应的刀模具");

        return bindTool(toolCode, type, tmToolMachines.get(0));
    }
}
