package com.bs.mes.schedule.strategy;

import cn.hutool.core.date.DateUtil;
import com.bsn.mes.pro.schedule.*;
import com.bsn.mes.pro.schedule.impl.BaoshenScheduleAlgorithmAdapter;
import com.bsn.mes.pro.schedule.strategy.EquipmentSelectionStrategyImpl;
import com.bsn.mes.pro.service.impl.ProArrangeServiceImpl;
import org.apache.tomcat.jni.Local;
import org.junit.Assert;
import org.junit.Test;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class EquipmentSelectionStrategyImplTest {
    @Test public void evaluateOptionalEquipment() throws Exception {
        EquipmentSelectionStrategyImpl equipmentSelectionStrategy = new EquipmentSelectionStrategyImpl();
        Method methods = EquipmentSelectionStrategyImpl.class.getDeclaredMethod("evaluateOptionalEquipment", Task.class, List.class);
        methods.setAccessible(true);
        Task task = new Task(1l, null);
        task.setOrderQuantity(BigDecimal.valueOf(750));
        List<IOperationTimeCalculator.OperationTimePlan> operationTimePlans = new ArrayList<IOperationTimeCalculator.OperationTimePlan>();
        IOperationTimeCalculator.OperationTimePlan plan1 = new IOperationTimeCalculator.OperationTimePlan();
        plan1.setScore(1);
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItems = new ArrayList<IOperationTimeCalculator.OperationTimePlanItem>();
        IOperationTimeCalculator.OperationTimePlanItem plan1Item1 = new IOperationTimeCalculator.OperationTimePlanItem(BigDecimal.valueOf(500),
                Duration.ofHours(2), Duration.ofHours(4), "白班");
        operationTimePlanItems.add(plan1Item1);
        plan1.setPlan(operationTimePlanItems);

        IOperationTimeCalculator.OperationTimePlan plan2 = new IOperationTimeCalculator.OperationTimePlan();
        plan1.setScore(1);
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItems2 = new ArrayList<IOperationTimeCalculator.OperationTimePlanItem>();
        IOperationTimeCalculator.OperationTimePlanItem plan2Item1 = new IOperationTimeCalculator.OperationTimePlanItem(BigDecimal.valueOf(500),
                Duration.ofHours(3), Duration.ofHours(5), "中班");
        operationTimePlanItems2.add(plan2Item1);
        plan2.setPlan(operationTimePlanItems2);

        operationTimePlans.add(plan1);
        operationTimePlans.add(plan2);

        List<EvaluateOptionalEquipmentResult> rst = (List<EvaluateOptionalEquipmentResult>)methods.invoke(equipmentSelectionStrategy, task, operationTimePlans);
        Assert.assertEquals(rst.size(), 2);
        Assert.assertEquals(Duration.ofHours(4), rst.get(0).getEndedTime());
        Assert.assertEquals(BigDecimal.valueOf(500), rst.get(0).getDetails().get(0).getQuantity());
        Assert.assertEquals(Duration.ofHours(4), rst.get(1).getEndedTime());
        Assert.assertEquals(BigDecimal.valueOf(250), rst.get(1).getDetails().get(0).getQuantity());
    }

    @Test public void evaluateOptionalEquipment2() throws Exception {
        LocalDateTime scheduleStartDate=LocalDateTime.of(2022,5,5,0,0);
        CalendarManager calendarManager = new CalendarManager();
        Equipment resource1 = new Equipment() {{
            setId(1l);
            setWorkCenterId(1l);
            setWorkUnitName("柔版印刷-1");
            setIsParallel(false);
            setCalendarManager(calendarManager);
        }};
        Equipment resource2 = new Equipment() {{
            setId(2l);
            setWorkCenterId(1l);
            setWorkUnitName("柔版印刷-2");
            setIsParallel(false);
            setCalendarManager(calendarManager);
        }};
        Equipment resource3 = new Equipment() {{
            setId(3l);
            setWorkCenterId(1l);
            setWorkUnitName("柔版印刷-2");
            setIsParallel(false);
            setCalendarManager(calendarManager);
        }};
        List<Equipment> equipmentList = new ArrayList<Equipment>();
        equipmentList.add(resource1);
        equipmentList.add(resource2);
        equipmentList.add(resource3);
        calendarManager.addWorkingCalendar(this.buildWorkingCalendar(resource1, scheduleStartDate, "22-05-05 12:30","22-05-05 16:30"));
        calendarManager.addWorkingCalendar(this.buildWorkingCalendar(resource2, scheduleStartDate, "22-05-05 13:30","22-05-05 17:30"));
        calendarManager.addWorkingCalendar(this.buildWorkingCalendar(resource2, scheduleStartDate, "22-05-06 13:30","22-05-05 17:30"));
        EquipmentSelectionStrategyImpl equipmentSelectionStrategy = new EquipmentSelectionStrategyImpl();
        Job job = new Job("1", Duration.ofHours(24));
        Task task = new Task(1l, job);
        task.setOrderQuantity(BigDecimal.valueOf(1000));
        task.setStandardWorkingTime(Duration.ofSeconds(20));
        task.setCalendarManager(calendarManager);
        task.setWorkCenterId(1l);
        task.setOptionalEquipments(equipmentList);
        task.setLimitedTimeCalculator(new BaoshenScheduleAlgorithmAdapter.OrderLimitedTimeCalculator());
        task.setOperationTimeCalculator(new BaoshenScheduleAlgorithmAdapter.OrderOperationTimeCalculator());
        List<EvaluateOptionalEquipmentResult> rst = equipmentSelectionStrategy.evaluateOptionalEquipment(task);
        Assert.assertEquals(2, rst.size());
        Assert.assertEquals(Duration.ofHours(12).plus(Duration.ofMinutes(30)).plus(Duration.ofHours(4)), rst.get(0).getEndedTime());
        Assert.assertEquals(Duration.ofHours(13).plus(Duration.ofMinutes(30)).plus(Duration.ofMinutes(94)), rst.get(1).getEndedTime());

    }

    private Calendar.WorkingCalendar buildWorkingCalendar(Resource resource, LocalDateTime scheduleStartDate, String start, String end){

        return new Calendar.WorkingCalendar() {{
            LocalDateTime startDate = DateUtil.toLocalDateTime(DateUtil.parse(start,"yy-MM-dd HH:mm"));
            LocalDateTime endDate = DateUtil.toLocalDateTime(DateUtil.parse(end,"yy-MM-dd HH:mm"));
            setStartedTime(Duration.between(scheduleStartDate, startDate));
            setEndedTime(Duration.between(scheduleStartDate, endDate));
            setResource(resource);
        }};
    }
}
