package com.bs.mes.schedule.strategy;

import com.bsn.mes.pro.schedule.IOperationTimeCalculator;
import com.bsn.mes.pro.schedule.Job;
import com.bsn.mes.pro.schedule.TaskSchedulingContext;
import com.bsn.mes.pro.schedule.strategy.EvaluateEquipmentDeliveryTime;
import com.bsn.mes.pro.schedule.strategy.EvaluateEquipmentShift;
import org.junit.Assert;
import org.junit.Test;

import java.time.Duration;
import java.util.ArrayList;
import java.util.List;

public class EvaluateEquipmentDeliveryTimeTest {
    @Test public void evaluate(){
        EvaluateEquipmentDeliveryTime evaluateEquipmentDeliveryTime = new EvaluateEquipmentDeliveryTime();
        TaskSchedulingContext context = new TaskSchedulingContext();
        IOperationTimeCalculator.OperationTimePlan operationTimePlan = new IOperationTimeCalculator.OperationTimePlan();
        List<IOperationTimeCalculator.OperationTimePlanItem> operationTimePlanItemList = new ArrayList<IOperationTimeCalculator.OperationTimePlanItem>();
        operationTimePlan.setPlan(operationTimePlanItemList);
        operationTimePlan.setScheduledSuccess(true);
        Job job = new Job("1", Duration.ofMinutes(12));
        job.setProducedDeliveryTime(Duration.ofMinutes(3000));
        context.setJob(job);
        operationTimePlanItemList.add(new IOperationTimeCalculator.OperationTimePlanItem(){{
            setStartedTime(Duration.ofMinutes(2888));
            setEndedTime(Duration.ofMinutes(2898));
            setShiftType("白班");
        }});
        //设备在交期范围内的加工
        double rst = evaluateEquipmentDeliveryTime.evaluate(context, operationTimePlan);
        Assert.assertEquals(3.0, rst, Double.valueOf(0.001));
        operationTimePlanItemList.add(new IOperationTimeCalculator.OperationTimePlanItem(){{
            setStartedTime(Duration.ofMinutes(3000));
            setEndedTime(Duration.ofMinutes(3002));
            setShiftType("白班");
        }});
        rst = evaluateEquipmentDeliveryTime.evaluate(context, operationTimePlan);
        Assert.assertEquals(-1.0, rst, Double.valueOf(0.001));
        //设备在交期范围外的加工
        operationTimePlanItemList.get(1).setShiftType("中班");
        rst = evaluateEquipmentDeliveryTime.evaluate(context, operationTimePlan);
        Assert.assertEquals(-1.0, rst, Double.valueOf(0.001));
        //累加上完成时间评分
        rst = evaluateEquipmentDeliveryTime.evaluate(context, operationTimePlan);
        Assert.assertEquals(-1.0+1.0/Duration.ofMinutes(3002).toMinutes(), rst, Double.valueOf(0.00000));
    }
}
